@@ -8,15 +8,16 @@ our @ISA = qw(Exporter);
 our @EXPORT_OK   = qw(rcsv1D rcsv2D wcsv1D wcsv2D);
 our %EXPORT_TAGS = (all => \@EXPORT_OK);
 
-our $VERSION = '0.006';
+our $VERSION = '0.007';
 
 use Config;
-use constant NO64BITINT => (($Config{use64bitint} || '') eq 'define' || $Config{longsize} >= 8) ? 0 : 1;
+use constant NO64BITINT => ($Config{ivsize} < 8) ? 1 : 0;
+use constant NODATETIME => eval { require PDL::DateTime; require Time::Moment; 1 } ? 0 : 1;
 use constant DEBUG      => $ENV{PDL_IO_CSV_DEBUG} ? 1 : 0;
 
 use PDL;
 use Text::CSV_XS;
-use Scalar::Util qw(looks_like_number openhandle);
+use Scalar::Util qw(looks_like_number openhandle blessed);
 
 use Carp;
 $Carp::Internal{ (__PACKAGE__) }++;
@@ -58,7 +59,7 @@ sub wcsv1D {
 
   my $bad2empty = $O->{bad2empty};
 
-  while (ref $_[0] eq 'PDL') {
+  while (blessed $_[0] && $_[0]->isa('PDL')) {
     $c_pdl[$cols] = shift;
     croak "FATAL: wcsv1D() expects 1D piddles" unless $c_pdl[$cols]->ndims == 1;
     $c_size[$cols]       = PDL::Core::howbig($c_pdl[$cols]->get_datatype);
@@ -89,10 +90,15 @@ sub wcsv1D {
       if ($c_max_offset[$c] >= $c_offset[$c]) {
         if ($bad2empty && $c_bad[$c]) {
           my $v = substr(${$c_dataref[$c]}, $c_offset[$c], $c_size[$c]);
-          $v[$c] = unpack($c_pack[$c], $v) if $v ne $c_bad[$c];
+          if ($v ne $c_bad[$c]) {
+            $v[$c] = unpack($c_pack[$c], $v);
+            $v[$c] = PDL::DateTime::ll2dt($v[$c]) if ref $c_pdl[$c] eq 'PDL::DateTime';
+          }
         }
         else {
-          $v[$c] = unpack($c_pack[$c], substr(${$c_dataref[$c]}, $c_offset[$c], $c_size[$c]));
+          my $v = substr(${$c_dataref[$c]}, $c_offset[$c], $c_size[$c]);
+          $v[$c] = unpack($c_pack[$c], $v);
+          $v[$c] = PDL::DateTime::ll2dt($v[$c]) if ref $c_pdl[$c] eq 'PDL::DateTime';
         }
       }
       $c_offset[$c] += $c_size[$c];
@@ -149,7 +155,7 @@ sub wcsv2D {
 sub rcsv1D {
   my ($fh, $coli, $O, $C) = _proc_rargs(@_);
 
-  my ($c_type, $c_pack, $c_sizeof, $c_pdl, $c_bad, $c_dataref, $c_idx, $allocated, $cols); # initialize after we get 1st line
+  my ($c_type, $c_pack, $c_sizeof, $c_pdl, $c_bad, $c_dataref, $c_idx, $c_dt, $allocated, $cols); # initialize after we get 1st line
 
   my $csv = Text::CSV_XS->new($C) or croak "" . Text::CSV_XS->error_diag();
   my $processed = 0;
@@ -161,7 +167,21 @@ sub rcsv1D {
 
   warn "Fetching 1D " . _dbg_msg($O, $C) . "\n" if $O->{debug};
   # skip headers
-  $csv->getline($fh) for (1..$O->{header});
+  my $headerline;
+  my $headerdetection;
+  if (looks_like_number($O->{header})) {
+    if ($O->{header} == 1) {
+      # get the header line (only if skipping exactly 1 line)
+      $headerline = $csv->getline($fh);
+    }
+    else {
+      $csv->getline($fh) for (1..$O->{header});
+    }
+  }
+  elsif ($O->{header} && $O->{header} eq 'auto') {
+    $headerdetection = 1;
+  }
+
   while (!$finished) {
     my $rows = 0;
     my @bytes;
@@ -169,8 +189,18 @@ sub rcsv1D {
     while ($rows < $chunk) {
       my $r = $csv->getline($fh);
       if (defined $r) {
+        if (defined $headerdetection) {
+          my $numeric = 0;
+          for (@$r) { $numeric++ if looks_like_number($_) || (!NODATETIME && defined PDL::DateTime::dt2ll($_)) }
+          if ($numeric == 0) {
+            # no numeric values found => skip this line but keep it as a potential header
+            $headerline = $r;
+            next;
+          }
+          $headerdetection = undef;
+        }
         unless (defined $c_type) {
-          ($c_type, $c_pack, $c_sizeof, $c_pdl, $c_bad, $c_dataref, $c_idx, $allocated, $cols) = _init_1D($coli, scalar @$r, $O);
+          ($c_type, $c_pack, $c_sizeof, $c_pdl, $c_bad, $c_dataref, $c_idx, $c_dt, $allocated, $cols) = _init_1D($coli, $r, $O);
           warn "Initialized size=$allocated, cols=$cols, type=".join(",",@$c_type)."\n" if $O->{debug};
         }
         if ($dec_comma) {
@@ -189,6 +219,19 @@ sub rcsv1D {
             }
           }
         }
+        if (defined $c_dt) {
+          for (0..$cols-1) {
+            next unless defined $c_dt->[$_];
+            my $v = PDL::DateTime::dt2ll($r->[$_]);
+            if (defined $v) {
+              $r->[$_] = $v;
+            }
+            else {
+              $r->[$_] = $c_bad->[$_];
+              $c_pdl->[$_]->badflag(1);
+            }
+          }
+        }
         if ($text2bad) {
           if (defined $coli) {
             for (0..$cols-1) {
@@ -242,13 +285,17 @@ sub rcsv1D {
   }
 
   #XXX close $fh;
-
   if (ref $c_pdl eq 'ARRAY') {
     if ($processed != $allocated) {
       warn "Reshape to: '$processed' (final)\n" if $O->{debug};
       $c_pdl->[$_]->reshape($processed) for (0..$cols-1);
     }
     $c_pdl->[$_]->upd_data for (0..$cols-1);
+    if (ref $headerline eq 'ARRAY') {
+      for (0..$cols-1) {
+        $c_pdl->[$_]->hdr->{col_name} = $headerline->[$_] if $headerline->[$_] && $headerline->[$_] ne '';
+      };
+    }
     return @$c_pdl;
   }
 
@@ -353,7 +400,7 @@ sub rcsv2D {
 
   #XXX close $fh;
 
-  if (ref $c_pdl eq 'PDL') {
+  if (blessed $c_pdl && $c_pdl->isa('PDL')) {
     if ($processed != $allocated) {
       warn "Reshaping to $processed (final)\n" if $O->{debug};
       $c_pdl->reshape($cols, $processed); # allocate the exact size
@@ -378,7 +425,7 @@ sub _dbg_msg {
 
 sub _proc_wargs {
   my $options        = ref $_[-1] eq 'HASH' ? pop : {};
-  my $filename_or_fh = ref $_[-1] ne 'PDL'  ? pop : undef;
+  my $filename_or_fh = !blessed $_[-1] || !$_[-1]->isa('PDL') ? pop : undef;
 
   my $C = { %$options }; # make a copy
 
@@ -392,7 +439,16 @@ sub _proc_wargs {
   $C->{eol}      = "\n" unless defined $C->{eol};
 
   if (defined $O->{header}) {
-    croak "FATAL: header should be arrayref" unless ref $O->{header} eq 'ARRAY';
+    croak "FATAL: header should be arrayref" unless ref $O->{header} eq 'ARRAY' || $O->{header} eq 'auto';
+    if ($O->{header} eq 'auto') {
+      my @n;
+      my $count = 0;
+      for (@_) {
+        push @n, my $n = $_->hdr->{col_name};
+        $count++ if defined $n;
+      }
+      $O->{header} = $count > 0 ? \@n : undef;
+    }
   }
 
   my $fh;
@@ -419,12 +475,13 @@ sub _proc_rargs {
   my $C = { %$options }; # make a copy
 
   # get options related to this module the rest will be passed to Text::CSV_XS
-  my @keys = qw/ reshape_inc fetch_chunk type debug empty2bad text2bad header decimal_comma encoding /;
+  my @keys = qw/ reshape_inc fetch_chunk type debug empty2bad text2bad header decimal_comma encoding detect_datetime /;
   my $O = { map { $_ => delete $C->{$_} } @keys };
   $O->{fetch_chunk} ||= 40_000;
   $O->{reshape_inc} ||= 80_000;
   $O->{type}        ||= double;
   $O->{header}      ||= 0;
+  ###$O->{detect_datetime} = 1 if !defined $O->{detect_datetime} && !NODATETIME;
   $O->{debug} = DEBUG unless defined $O->{debug};
 
   # reshape_inc cannot be lower than fetch_chunk
@@ -454,8 +511,8 @@ sub _proc_rargs {
 }
 
 sub _init_1D {
-  my ($coli, $colcount, $O) = @_;
-
+  my ($coli, $firstline, $O) = @_;
+  my $colcount = scalar @$firstline;
   my $cols;
   if (!defined $coli) {    # take all columns
     $cols = $colcount;
@@ -476,11 +533,28 @@ sub _init_1D {
 
   if (ref $O->{type} eq 'ARRAY') {
     @c_type = @{$O->{type}};
+    push @c_type, ((double) x ($cols - @c_type));
   }
   else {
     $c_type[$_] = $O->{type} for (0..$cols-1);
   }
 
+  my @c_dt;
+  for (0..$cols-1) {
+    if ($c_type[$_] eq 'datetime') {
+      croak "PDL::DateTime not installed" if NODATETIME;
+      $c_type[$_] = longlong;
+      $c_dt[$_] = 'datetime';
+    }
+    elsif ($O->{detect_datetime}) {
+      croak "PDL::DateTime not installed" if NODATETIME;
+      if (defined PDL::DateTime::dt2ll($firstline->[$_])) {
+        $c_type[$_] = longlong;
+        $c_dt[$_] = 'datetime';
+      }
+    }
+  }
+
   my $allocated = $O->{reshape_inc};
   for (0..$cols-1) {
     $c_type[$_] = double if !defined $c_type[$_];
@@ -488,16 +562,15 @@ sub _init_1D {
     croak "FATAL: your perl does not support 64bitint (avoid using type longlong)" if $c_pack[$_] eq 'q' && NO64BITINT;
     croak "FATAL: invalid type '$c_type[$_]' for column $_" if !$c_pack[$_];
     $c_sizeof[$_] = length pack($c_pack[$_], 1);
-    $c_pdl[$_] = zeroes($c_type[$_], $allocated);
+    $c_pdl[$_] = $c_dt[$_] ? PDL::DateTime->new(zeroes(longlong, $allocated)) : zeroes($c_type[$_], $allocated);
     $c_dataref[$_] = $c_pdl[$_]->get_dataref;
     $c_bad[$_] = $c_pdl[$_]->badvalue;
     $c_idx[$_] = 0;
-
     my $big = PDL::Core::howbig($c_pdl[$_]->get_datatype);
     croak "FATAL: column $_ mismatch (type=$c_type[$_], sizeof=$c_sizeof[$_], big=$big)" if $big != $c_sizeof[$_];
   }
 
-  return (\@c_type, \@c_pack, \@c_sizeof, \@c_pdl, \@c_bad, \@c_dataref, \@c_idx, $allocated, $cols);
+  return (\@c_type, \@c_pack, \@c_sizeof, \@c_pdl, \@c_bad, \@c_dataref, \@c_idx, (@c_dt > 0 ? \@c_dt : undef), $allocated, $cols);
 }
 
 sub _init_2D {
@@ -530,6 +603,17 @@ sub _init_2D {
   return ($c_type, $c_pack, $c_sizeof, $c_pdl, $c_bad, $c_dataref, $allocated, $cols);
 }
 
+sub _fix_datetime_value {
+  my $v = shift;
+  # '2015-12-29' > '2015-12-29T00Z'
+  return $v."T00Z" if $v =~ /^\d\d\d\d-\d\d-\d\d$/;
+  # '2015-12-29 11:59' > '2015-12-29 11:59Z'
+  return $v."Z"    if $v =~ /^\d\d\d\d-\d\d-\d\d[ T]\d\d:\d\d$/;
+  # '2015-12-29 11:59:11' > '2015-12-29 11:59:11Z' or '2015-12-29 11:59:11.123' > '2015-12-29 11:59:11.123Z'
+  return $v."Z"    if $v =~ /^\d\d\d\d-\d\d-\d\d[ T]\d\d:\d\d:\d\d(\.\d+)?$/;
+  return $v;
+}
+
 1;
 
 __END__
@@ -618,6 +702,20 @@ or separately for each column/piddle:
 
   my ($a, $b, $c) = rcsv1D($csv, {type => [long, double, double]});
 
+Special datetime handling (you need to have L<PDL::DateTime> installed):
+
+  my ($a, $b, $c) = rcsv1D($csv, {type => [long, 'datetime', double]});
+  # piddle $b will be an instance of PDL::DateTime
+
+or you cat let PDL::IO::CSV try to detect datetime columns (detection is based only on the first csv line)
+
+  my ($a, $b, $c) = rcsv1D($csv, {detect_datetime=>1});
+
+=item * detect_datetime
+
+Values C<0> (default) or C<1>. Try to detect datetime columns, corresponding output piddles will be
+instances of L<PDL::Datetime> (which you need to have installed).
+
 =item * fetch_chunk
 
 We do not try to load all CSV data into memory at once; we load them in chunks defined by this parameter.
@@ -647,6 +745,11 @@ values are silently converted into C<0>.
 Values C<0> (default) or C<N> (positive integer) - consider the first C<N> lines as headers and skip them.
 BEWARE: we are talking here about skipping CSV lines which in some cases might be more than 1 text line.
 
+NOTE: header values (if any) are considered to be column names and are stored in loaded piddles in $pdl->hdr->{col_name}
+
+NOTE: C<rcsv1D> accepts a special C<header> value C<'auto'> which skips rows (from beginning) that have
+in all columns non-numeric values.
+
 =item * decimal_comma
 
 Values C<0> (default) or C<1> - accept C<,> (comma) as a decimal separator (there is a performance cost when turned on).
@@ -696,6 +799,8 @@ Saves data from one or more 1D piddles to CSV file.
   # but also as a piddle method
   $pdl1D->wcsv1D("file.csv");
 
+NOTE: piddles piddles are instances of L<PDL::DateTime> are exported by wcsv1D as ISO 8601 strings.
+
 Parameters:
 
 =over
@@ -716,7 +821,8 @@ Items supported in B<options> hash:
 
 =item * header
 
-Arrayref with values that will be printed as the first CSV line.
+Arrayref with values that will be printed as the first CSV line. Or C<'auto'> value which means that column
+names are taken from $pdl->hdr->{col_name}.
 
 =item * bad2empty
 
